package defi

import (
	"io"
	"net/http"
	"net/url"
	"os"
	"strconv"

	"birdcall/internal/consts"
	"birdcall/internal/library/birdeye"
	"encoding/json"
)

type Defi struct {
	Address          string  ` json:"address" dc:"代币地址"`
	Chain            string  ` json:"chain" dc:"链名"`
	CheckLiquidity   float64 ` json:"check_liquidity" dc:"流动性验证"`
	IncludeLiquidity bool    ` json:"include_liquidity" dc:"是否返回流动数据" `
}

func (d *Defi) Support() (*birdeye.CommonRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(consts.DEFI_URL)
	fullURL := baseURL + consts.NET_WORKS

	apiReq, err := http.NewRequest("GET", fullURL, nil)
	if err != nil {
		return nil, err
	}
	//构建请求头
	apiReq = birdeye.MakeHeader(apiReq, d.Chain)

	apiRes, err := http.DefaultClient.Do(apiReq)
	if err != nil {
		return nil, err
	}

	//处理结果
	body, _ := io.ReadAll(apiRes.Body)
	var commonRes *birdeye.CommonRes
	err = json.Unmarshal(body, &commonRes)
	if err != nil {
		return nil, err
	}
	return commonRes, nil
}

func (d *Defi) Price() (*birdeye.CommonRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(consts.DEFI_URL)
	params := url.Values{}
	params.Add(consts.CheckLiquidity, strconv.FormatFloat(d.CheckLiquidity, 'f', 2, 64))
	params.Add(consts.IncludeLiquidity, strconv.FormatBool(d.IncludeLiquidity))
	params.Add(consts.ADDRESS, d.Address)
	fullURL := baseURL + consts.PRICE + consts.QUESTION_MARK + params.Encode()

	apiReq, err := http.NewRequest(consts.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}

	//构建请求头
	apiReq = birdeye.MakeHeader(apiReq, d.Chain)

	//发送请求
	apiRes, err := http.DefaultClient.Do(apiReq)
	if err != nil {
		return nil, err
	}

	//处理结果
	body, _ := io.ReadAll(apiRes.Body)
	var commonRes *birdeye.CommonRes
	err = json.Unmarshal(body, &commonRes)
	if err != nil {
		return nil, err
	}
	return commonRes, nil
}
